@@ -21,6 +21,7 @@ static MVMObject * index_mapping_and_flat_list(MVMThreadContext *tc, MVMObject *
     MVM_gc_root_temp_push(tc, (MVMCollectable **)&st);
     MVM_gc_root_temp_push(tc, (MVMCollectable **)&mro);
 
+    /* We're only ever called with tc->allocate_in_gen2 set, so no need to MVMROOT here. */
     flat_list = MVM_repr_alloc_init(tc, MVM_hll_current(tc)->slurpy_array_type);
 
     class_list = MVM_repr_alloc_init(tc, MVM_hll_current(tc)->slurpy_array_type);
@@ -38,12 +39,16 @@ static MVMObject * index_mapping_and_flat_list(MVMThreadContext *tc, MVMObject *
         MVMObject *parents     = MVM_repr_at_pos_o(tc, type_info, 2);
         MVMint32  num_parents = MVM_repr_elems(tc, parents);
 
-        MVM_gc_root_temp_push(tc, (MVMCollectable **)&current_class);
         if (num_parents <= 1) {
             /* Get attributes and iterate over them. */
             MVMObject *attributes     = MVM_repr_at_pos_o(tc, type_info, 1);
-            MVMIter * const attr_iter = (MVMIter *)MVM_iter(tc, attributes);
             MVMObject *attr_map = NULL;
+            MVMIter   *attr_iter;
+
+            /* Add to class list firstly, so we can avoid a MVMROOT. */
+            MVM_repr_push_o(tc, class_list, current_class);
+
+            attr_iter = (MVMIter *)MVM_iter(tc, attributes);
 
             if (MVM_iter_istrue(tc, attr_iter)) {
                 attr_map = MVM_repr_alloc_init(tc, MVM_hll_current(tc)->slurpy_hash_type);
@@ -71,8 +76,7 @@ static MVMObject * index_mapping_and_flat_list(MVMThreadContext *tc, MVMObject *
                 MVM_repr_push_o(tc, flat_list, attr);
             }
 
-            /* Add to class list and map list. */
-            MVM_repr_push_o(tc, class_list, current_class);
+            /* Add to map list. */
             MVM_repr_push_o(tc, attr_map_list, attr_map);
         }
         else {
@@ -80,8 +84,6 @@ static MVMObject * index_mapping_and_flat_list(MVMThreadContext *tc, MVMObject *
             MVM_exception_throw_adhoc(tc,
                 "CStruct representation does not support multiple inheritance");
         }
-
-        MVM_gc_root_temp_pop(tc); /* current_class */
     }
 
     MVM_gc_root_temp_pop_n(tc, 2); /* mro, st */
@@ -115,9 +117,10 @@ static MVMint32 round_up_to_multi(MVMint32 i, MVMint32 m) {
 static void compute_allocation_strategy(MVMThreadContext *tc, MVMObject *repr_info, MVMCStructREPRData *repr_data, MVMSTable *st) {
     /* Compute index mapping table and get flat list of attributes. */
     MVMObject *flat_list;
-    MVM_gc_root_temp_push(tc, (MVMCollectable **)&st);
-    MVM_gc_root_temp_push(tc, (MVMCollectable **)&repr_info);
-    flat_list = index_mapping_and_flat_list(tc, repr_info, repr_data, st);
+
+    MVMROOT(tc, st) {
+        flat_list = index_mapping_and_flat_list(tc, repr_info, repr_data, st);
+    }
 
     /* If we have no attributes in the index mapping, then just the header. */
     if (repr_data->name_to_index_mapping[0].class_key == NULL) {
@@ -180,6 +183,7 @@ static void compute_allocation_strategy(MVMThreadContext *tc, MVMObject *repr_in
                 MVMint32  type_id    = REPR(type)->ID;
                 if (spec->inlineable == MVM_STORAGE_SPEC_INLINED &&
                         (spec->boxed_primitive == MVM_STORAGE_SPEC_BP_INT ||
+                         spec->boxed_primitive == MVM_STORAGE_SPEC_BP_UINT64 ||
                          spec->boxed_primitive == MVM_STORAGE_SPEC_BP_NUM)) {
                     /* It's a boxed int or num; pretty easy. It'll just live in the
                      * body of the struct. Instead of masking in i here (which
@@ -357,8 +361,6 @@ static void compute_allocation_strategy(MVMThreadContext *tc, MVMObject *repr_in
         if (repr_data->initialize_slots)
             repr_data->initialize_slots[cur_init_slot] = -1;
     }
-
-    MVM_gc_root_temp_pop_n(tc, 2); /* repr_info, st */
 }
 
 /* Helper for reading a pointer at the specified offset. */
@@ -395,11 +397,11 @@ static MVMint32 try_get_slot(MVMThreadContext *tc, MVMCStructREPRData *repr_data
 static MVMObject * type_object_for(MVMThreadContext *tc, MVMObject *HOW) {
     MVMSTable *st  = MVM_gc_allocate_stable(tc, &CStruct_this_repr, HOW);
 
-    MVMROOT(tc, st, {
+    MVMROOT(tc, st) {
         MVMObject *obj = MVM_gc_allocate_type_object(tc, st);
         MVM_ASSIGN_REF(tc, &(st->header), st->WHAT, obj);
         st->size = sizeof(MVMCStruct);
-    });
+    }
 
     return st->WHAT;
 }
@@ -421,7 +423,7 @@ static void initialize(MVMThreadContext *tc, MVMSTable *st, MVMObject *root, voi
 
     /* Allocate object body. */
     MVMCStructBody *body = (MVMCStructBody *)data;
-    body->cstruct = MVM_calloc(1, repr_data->struct_size > 0 ? repr_data->struct_size : 1);
+    body->cstruct = calloc(1, repr_data->struct_size > 0 ? repr_data->struct_size : 1);
 
     /* Allocate child obj array. */
     if (repr_data->num_child_objs > 0)
@@ -487,7 +489,7 @@ static void get_attribute(MVMThreadContext *tc, MVMSTable *st, MVMObject *root,
                 MVMObject *obj     = body->child_objs[real_slot];
                 if (!obj) {
                     /* No cached object. */
-                    MVMROOT(tc, root, {
+                    MVMROOT(tc, root) {
                         //make it if it's inlined
                         if (repr_data->attribute_locations[slot] & MVM_CSTRUCT_ATTR_INLINED) {
                             if (type == MVM_CSTRUCT_ATTR_CARRAY) {
@@ -527,18 +529,18 @@ static void get_attribute(MVMThreadContext *tc, MVMSTable *st, MVMObject *root,
                                     obj = MVM_nativecall_make_cpointer(tc, typeobj, cobj);
                                 }
                                 else if(type == MVM_CSTRUCT_ATTR_STRING) {
-                                    MVMROOT(tc, typeobj, {
+                                    MVMROOT(tc, typeobj) {
                                         MVMString *str = MVM_string_utf8_decode(tc, tc->instance->VMString,
                                             cobj, strlen(cobj));
                                         obj = MVM_repr_box_str(tc, typeobj, str);
-                                    });
+                                    }
                                 }
                             }
                             else {
                                 obj = typeobj;
                             }
                         }
-                    });
+                    }
                     MVM_ASSIGN_REF(tc, &(root->header), ((MVMCStruct*)root)->body.child_objs[real_slot], obj);
                     }
                 result_reg->o = obj;
@@ -553,6 +555,14 @@ static void get_attribute(MVMThreadContext *tc, MVMSTable *st, MVMObject *root,
                 MVM_exception_throw_adhoc(tc, "CStruct: invalid native get of object attribute");
             break;
         }
+        case MVM_reg_uint64: {
+            if (attr_st)
+                result_reg->u64 = attr_st->REPR->box_funcs.get_uint(tc, attr_st, root,
+                    ((char *)body->cstruct) + repr_data->struct_offsets[slot]);
+            else
+                MVM_exception_throw_adhoc(tc, "CStruct: invalid native get of object attribute");
+            break;
+        }
         case MVM_reg_num64: {
             if (attr_st)
                 result_reg->n64 = attr_st->REPR->box_funcs.get_num(tc, attr_st, root,
@@ -634,7 +644,7 @@ static void bind_attribute(MVMThreadContext *tc, MVMSTable *st, MVMObject *root,
                             break;
                         }
                         else
-                            cobj = ((MVMCStruct *)value)->body.cstruct; 
+                            cobj = ((MVMCStruct *)value)->body.cstruct;
                     }
                     else if (type == MVM_CSTRUCT_ATTR_CPPSTRUCT) {
                         if (REPR(value)->ID != MVM_REPR_ID_MVMCPPStruct)
@@ -676,6 +686,14 @@ static void bind_attribute(MVMThreadContext *tc, MVMSTable *st, MVMObject *root,
                 MVM_exception_throw_adhoc(tc, "CStruct: invalid native binding to object attribute");
             break;
         }
+        case MVM_reg_uint64: {
+            if (attr_st)
+                attr_st->REPR->box_funcs.set_uint(tc, attr_st, root,
+                    ((char *)body->cstruct) + repr_data->struct_offsets[slot], value_reg.u64);
+            else
+                MVM_exception_throw_adhoc(tc, "CStruct: invalid native binding to object attribute");
+            break;
+        }
         case MVM_reg_num64: {
             if (attr_st)
                 attr_st->REPR->box_funcs.set_num(tc, attr_st, root,
@@ -780,7 +798,7 @@ static void gc_cleanup(MVMThreadContext *tc, MVMSTable *st, void *data) {
     /* XXX For some reason, this causes crashes at the moment. Need to
      * work out why. */
     /*if (body->cstruct)
-        MVM_free(body->cstruct);*/
+        free(body->cstruct);*/
 }
 
 /* Called by the VM in order to free memory associated with this object. */
@@ -803,7 +821,7 @@ static MVMuint64 unmanaged_size(MVMThreadContext *tc, MVMSTable *st, void *data)
     MVMCStructREPRData *repr_data = (MVMCStructREPRData *)st->REPR_data;
     MVMuint64 result = 0;
 
-    /* The allocated (or just-poisted-at) memory block */
+    /* The allocated (or just-pointed-at) memory block */
     /* TODO make sure when structs properly track "ownership" to
      *      not add this for "child" structs */
     result += repr_data->struct_size;
@@ -865,9 +883,7 @@ static void deserialize_repr_data(MVMThreadContext *tc, MVMSTable *st, MVMSerial
     MVMint32 i, num_classes, num_slots;
 
     repr_data->struct_size = MVM_serialization_read_int(tc, reader);
-    if (reader->root.version >= 17) {
-        repr_data->struct_align = MVM_serialization_read_int(tc, reader);
-    }
+    repr_data->struct_align = MVM_serialization_read_int(tc, reader);
     repr_data->num_attributes = MVM_serialization_read_int(tc, reader);
     repr_data->num_child_objs = MVM_serialization_read_int(tc, reader);
 
