@@ -14,11 +14,11 @@ MVM_STATIC_INLINE MVMString * get_string_key(MVMThreadContext *tc, MVMObject *ke
 static MVMObject * type_object_for(MVMThreadContext *tc, MVMObject *HOW) {
     MVMSTable *st = MVM_gc_allocate_stable(tc, &MVMHash_this_repr, HOW);
 
-    MVMROOT(tc, st, {
+    MVMROOT(tc, st) {
         MVMObject *obj = MVM_gc_allocate_type_object(tc, st);
         MVM_ASSIGN_REF(tc, &(st->header), st->WHAT, obj);
         st->size = sizeof(MVMHash);
-    });
+    }
 
     return st->WHAT;
 }
@@ -30,20 +30,19 @@ static void copy_to(MVMThreadContext *tc, MVMSTable *st, void *src, MVMObject *d
 
     MVMStrHashTable *src_hashtable = &(src_body->hashtable);
     MVMStrHashTable *dest_hashtable = &(dest_body->hashtable);
-    if (MVM_str_hash_entry_size(tc, dest_hashtable)) {
+
+    if (dest_hashtable->table) {
         /* copy_to is, on reference types, only ever used as part of clone, and
          * that will always target a freshly created object.
          * So this should be unreachable. */
         MVM_oops(tc, "copy_to on MVMHash that is already initialized");
     }
-    MVM_str_hash_build(tc, dest_hashtable, sizeof(MVMHashEntry),
-                       MVM_str_hash_count(tc, src_hashtable));
-    MVMStrHashIterator iterator = MVM_str_hash_first(tc, src_hashtable);
-    while (!MVM_str_hash_at_end(tc, src_hashtable, iterator)) {
-        MVMHashEntry *entry = MVM_str_hash_current_nocheck(tc, src_hashtable, iterator);
-        MVMHashEntry *new_entry = MVM_str_hash_insert_nocheck(tc, dest_hashtable, entry->hash_handle.key);
-        MVM_ASSIGN_REF(tc, &(dest_root->header), new_entry->value, entry->value);
-        MVM_gc_write_barrier(tc, &(dest_root->header), &(new_entry->hash_handle.key->common.header));
+    MVM_str_hash_shallow_copy(tc, src_hashtable, dest_hashtable);
+    MVMStrHashIterator iterator = MVM_str_hash_first(tc, dest_hashtable);
+    while (!MVM_str_hash_at_end(tc, dest_hashtable, iterator)) {
+        MVMHashEntry *entry = MVM_str_hash_current_nocheck(tc, dest_hashtable, iterator);
+        MVM_gc_write_barrier(tc, &(dest_root->header), &(entry->value->header));
+        MVM_gc_write_barrier(tc, &(dest_root->header), &(entry->hash_handle.key->common.header));
         iterator = MVM_str_hash_next_nocheck(tc, src_hashtable, iterator);
     }
 }
@@ -52,7 +51,13 @@ static void copy_to(MVMThreadContext *tc, MVMSTable *st, void *src, MVMObject *d
 static void MVMHash_gc_mark(MVMThreadContext *tc, MVMSTable *st, void *data, MVMGCWorklist *worklist) {
     MVMHashBody     *body = (MVMHashBody *)data;
     MVMStrHashTable *hashtable = &(body->hashtable);
-    MVM_gc_worklist_presize_for(tc, worklist, 2 * MVM_str_hash_count(tc, hashtable));
+    MVMuint64            elems = MVM_str_hash_count(tc, hashtable);
+
+    /* Aren't holding anything, nothing to do. */
+    if (elems == 0)
+        return;
+
+    MVM_gc_worklist_presize_for(tc, worklist, 2 * elems);
     if (worklist->include_gen2) {
         MVMStrHashIterator iterator = MVM_str_hash_first(tc, hashtable);
         while (!MVM_str_hash_at_end(tc, hashtable, iterator)) {
@@ -182,16 +187,18 @@ static void deserialize(MVMThreadContext *tc, MVMSTable *st, MVMObject *root, vo
         MVM_oops(tc, "deserialize on MVMHash that is already initialized");
     }
     MVMint64 elems = MVM_serialization_read_int(tc, reader);
-    MVM_str_hash_build(tc, hashtable, sizeof(MVMHashEntry), elems);
-    MVMint64 i;
-    for (i = 0; i < elems; i++) {
-        MVMString *key = MVM_serialization_read_str(tc, reader);
-        if (!MVM_str_hash_key_is_valid(tc, key)) {
-            MVM_str_hash_key_throw_invalid(tc, key);
-        }
-        MVMObject *value = MVM_serialization_read_ref(tc, reader);
-        MVMHashEntry *entry = MVM_str_hash_insert_nocheck(tc, hashtable, key);
-        MVM_ASSIGN_REF(tc, &(root->header), entry->value, value);
+    if (elems) {
+        MVM_str_hash_build(tc, hashtable, sizeof(MVMHashEntry), elems);
+        MVMint64 i = 0;
+        do {
+            MVMString *key = MVM_serialization_read_str(tc, reader);
+            if (!MVM_str_hash_key_is_valid(tc, key)) {
+                MVM_str_hash_key_throw_invalid(tc, key);
+            }
+            MVMObject *value = MVM_serialization_read_ref(tc, reader);
+            MVMHashEntry *entry = MVM_str_hash_insert_nocheck(tc, hashtable, key);
+            MVM_ASSIGN_REF(tc, &(root->header), entry->value, value);
+        } while (++i < elems);
     }
 }
 
@@ -255,7 +262,7 @@ static void spesh(MVMThreadContext *tc, MVMSTable *st, MVMSpeshGraph *g, MVMSpes
 static MVMuint64 unmanaged_size(MVMThreadContext *tc, MVMSTable *st, void *data) {
     MVMHashBody *body = (MVMHashBody *)data;
 
-    return sizeof(MVMHashEntry) * MVM_str_hash_count(tc, &(body->hashtable));
+    return MVM_str_hash_allocated_size(tc, &(body->hashtable));
 }
 
 /* Initializes the representation. */
