@@ -6,11 +6,8 @@ MVM_STATIC_INLINE void hash_demolish_internal(MVMThreadContext *tc,
                                               struct MVMIndexHashTableControl *control) {
     size_t allocated_items = MVM_index_hash_allocated_items(control);
     size_t entries_size = MVM_hash_round_size_up(sizeof(struct MVMIndexHashEntry) * allocated_items);
-    size_t metadata_size = MVM_hash_round_size_up(allocated_items + 1);
-    size_t total_size
-        = entries_size + sizeof(struct MVMIndexHashTableControl) + metadata_size;
     char *start = (char *)control - entries_size;
-    MVM_fixed_size_free(tc, tc->instance->fsa, total_size, start);
+    MVM_free(start);
 }
 
 /* Frees the entire contents of the hash, leaving you just the hashtable itself,
@@ -26,7 +23,6 @@ void MVM_index_hash_demolish(MVMThreadContext *tc, MVMIndexHashTable *hashtable)
 
 
 MVM_STATIC_INLINE struct MVMIndexHashTableControl *hash_allocate_common(MVMThreadContext *tc,
-                                                                        MVMuint8 key_right_shift,
                                                                         MVMuint8 official_size_log2) {
     MVMuint32 official_size = 1 << (MVMuint32)official_size_log2;
     MVMuint32 max_items = official_size * MVM_INDEX_HASH_LOAD_FACTOR;
@@ -54,7 +50,7 @@ MVM_STATIC_INLINE struct MVMIndexHashTableControl *hash_allocate_common(MVMThrea
     assert(total_size == MVM_hash_round_size_up(total_size));
 
     struct MVMIndexHashTableControl *control =
-        (struct MVMIndexHashTableControl *) ((char *)MVM_fixed_size_alloc(tc, tc->instance->fsa, total_size) + entries_size);
+        (struct MVMIndexHashTableControl *) ((char *)MVM_malloc(total_size) + entries_size);
 
     control->official_size_log2 = official_size_log2;
     control->max_items = max_items;
@@ -64,7 +60,8 @@ MVM_STATIC_INLINE struct MVMIndexHashTableControl *hash_allocate_common(MVMThrea
     MVMuint8 initial_probe_distance = (1 << (8 - MVM_HASH_INITIAL_BITS_IN_METADATA)) - 1;
     control->max_probe_distance = max_probe_distance_limit > initial_probe_distance ? initial_probe_distance : max_probe_distance_limit;
     control->max_probe_distance_limit = max_probe_distance_limit;
-    control->key_right_shift = key_right_shift;
+    MVMuint8 bucket_right_shift = 8 * sizeof(MVMuint64) - official_size_log2;
+    control->key_right_shift = bucket_right_shift - control->metadata_hash_bits;
 
     MVMuint8 *metadata = (MVMuint8 *)(control + 1);
     memset(metadata, 0, metadata_size);
@@ -88,9 +85,7 @@ void MVM_index_hash_build(MVMThreadContext *tc,
         }
     }
 
-    hashtable->table = hash_allocate_common(tc,
-                                            (8 * sizeof(MVMuint64) - initial_size_base2),
-                                            initial_size_base2);
+    hashtable->table = hash_allocate_common(tc, initial_size_base2);
 }
 
 MVM_STATIC_INLINE void hash_insert_internal(MVMThreadContext *tc,
@@ -138,7 +133,7 @@ MVM_STATIC_INLINE void hash_insert_internal(MVMThreadContext *tc,
                 } while (old_probe_distance);
 
                 MVMuint32 entries_to_move = find_me_a_gap - ls.metadata;
-                size_t size_to_move = ls.entry_size * entries_to_move;
+                size_t size_to_move = (size_t) ls.entry_size * entries_to_move;
                 /* When we had entries *ascending* this was
                  * memmove(entry_raw + sizeof(struct MVMIndexHashEntry), entry_raw,
                  *         sizeof(struct MVMIndexHashEntry) * entries_to_move);
@@ -238,6 +233,7 @@ static struct MVMIndexHashTableControl *maybe_grow_hash(MVMThreadContext *tc,
         } while (--loop_count);
         assert(control->metadata_hash_bits);
         --control->metadata_hash_bits;
+        ++control->key_right_shift;
 
         control->max_probe_distance = new_probe_distance;
         /* Reset this to its proper value. */
@@ -252,9 +248,7 @@ static struct MVMIndexHashTableControl *maybe_grow_hash(MVMThreadContext *tc,
 
     struct MVMIndexHashTableControl *control_orig = control;
 
-    control = hash_allocate_common(tc,
-                                   control_orig->key_right_shift - 1,
-                                   control_orig->official_size_log2 + 1);
+    control = hash_allocate_common(tc, control_orig->official_size_log2 + 1);
 
     MVMuint8 *entry_raw = entry_raw_orig;
     MVMuint8 *metadata = metadata_orig;
