@@ -121,7 +121,7 @@ static MVMuint64 get_string_index(MVMThreadContext *tc, MVMHeapSnapshotState *ss
         &(col->alloc_strings_free), sizeof(char));
     col->strings_free[col->num_strings_free] = str_mode != STR_MODE_CONST;
     col->num_strings_free++;
-    col->strings[col->num_strings] = str_mode == STR_MODE_DUP ? strdup(str) : str;
+    col->strings[col->num_strings] = str_mode == STR_MODE_DUP ? MVM_strdup(str) : str;
     return col->num_strings++;
 }
 
@@ -514,9 +514,6 @@ static void process_workitems(MVMThreadContext *tc, MVMHeapSnapshotState *ss) {
                 process_collectable(tc, ss, &col, (MVMCollectable *)st, &sc_cache);
                 set_type_index(tc, ss, &col, st);
 
-                MVM_profile_heap_add_collectable_rel_const_cstr(tc, ss,
-                    (MVMCollectable *)st->method_cache, "Method cache");
-
                 for (i = 0; i < st->type_check_cache_length; i++)
                     MVM_profile_heap_add_collectable_rel_const_cstr_cached(tc, ss,
                         (MVMCollectable *)st->type_check_cache[i], "Type cache entry", &cache_1);
@@ -531,27 +528,6 @@ static void process_workitems(MVMThreadContext *tc, MVMHeapSnapshotState *ss) {
                         (MVMCollectable *)st->boolification_spec->method,
                         "Boolification method", &cache_2);
 
-                if (st->invocation_spec) {
-                    MVM_profile_heap_add_collectable_rel_const_cstr(tc, ss,
-                        (MVMCollectable *)st->invocation_spec->class_handle,
-                        "Invocation spec class handle");
-                    MVM_profile_heap_add_collectable_rel_const_cstr(tc, ss,
-                        (MVMCollectable *)st->invocation_spec->attr_name,
-                        "Invocation spec attribute name");
-                    MVM_profile_heap_add_collectable_rel_const_cstr(tc, ss,
-                        (MVMCollectable *)st->invocation_spec->invocation_handler,
-                        "Invocation spec invocation handler");
-                    MVM_profile_heap_add_collectable_rel_const_cstr(tc, ss,
-                        (MVMCollectable *)st->invocation_spec->md_class_handle,
-                        "Invocation spec class handle (multi)");
-                    MVM_profile_heap_add_collectable_rel_const_cstr(tc, ss,
-                        (MVMCollectable *)st->invocation_spec->md_cache_attr_name,
-                        "Invocation spec cache attribute name (multi)");
-                    MVM_profile_heap_add_collectable_rel_const_cstr(tc, ss,
-                        (MVMCollectable *)st->invocation_spec->md_valid_attr_name,
-                        "Invocation spec valid attribute name (multi)");
-                }
-
                 MVM_profile_heap_add_collectable_rel_const_cstr_cached(tc, ss,
                     (MVMCollectable *)st->WHO, "WHO", &cache_3);
                 MVM_profile_heap_add_collectable_rel_const_cstr(tc, ss,
@@ -560,9 +536,6 @@ static void process_workitems(MVMThreadContext *tc, MVMHeapSnapshotState *ss) {
                     (MVMCollectable *)st->HOW, "HOW");
                 MVM_profile_heap_add_collectable_rel_const_cstr(tc, ss,
                     (MVMCollectable *)st->HOW_sc, "HOW serialization context");
-                MVM_profile_heap_add_collectable_rel_const_cstr(tc, ss,
-                    (MVMCollectable *)st->method_cache_sc,
-                    "Method cache serialization context");
 
                 if (st->mode_flags & MVM_PARAMETRIC_TYPE) {
                     MVM_profile_heap_add_collectable_rel_const_cstr(tc, ss,
@@ -641,19 +614,6 @@ static void process_workitems(MVMThreadContext *tc, MVMHeapSnapshotState *ss) {
 
                 if (frame->extra) {
                     MVMFrameExtra *e = frame->extra;
-                    if (e->special_return_data && e->mark_special_return_data) {
-                        e->mark_special_return_data(tc, frame, ss->gcwl);
-                        process_gc_worklist(tc, ss, "Special return data");
-                    }
-                    if (e->continuation_tags) {
-                        MVMContinuationTag *tag = e->continuation_tags;
-                        while (tag) {
-                            MVM_profile_heap_add_collectable_rel_const_cstr(tc, ss,
-                                (MVMCollectable *)tag->tag, "Continuation tag");
-                            col.unmanaged_size += sizeof(MVMContinuationTag);
-                            tag = tag->next;
-                        }
-                    }
                     MVM_profile_heap_add_collectable_rel_const_cstr(tc, ss,
                         (MVMCollectable *)e->dynlex_cache_name,
                         "Dynamic lexical cache name");
@@ -749,13 +709,13 @@ static void process_workitems(MVMThreadContext *tc, MVMHeapSnapshotState *ss) {
  * relation to the current collectable with a constant C string that we
  * should not free. */
 void MVM_profile_heap_add_collectable_rel_const_cstr(MVMThreadContext *tc,
-        MVMHeapSnapshotState *ss, MVMCollectable *collectable, char *desc) {
+        MVMHeapSnapshotState *ss, MVMCollectable *collectable, const char *desc) {
     if (collectable)
         add_reference_const_cstr(tc, ss, desc,
             get_collectable_idx(tc, ss, collectable));
 }
 void MVM_profile_heap_add_collectable_rel_const_cstr_cached(MVMThreadContext *tc,
-        MVMHeapSnapshotState *ss, MVMCollectable *collectable, char *desc, MVMuint64 *cache) {
+        MVMHeapSnapshotState *ss, MVMCollectable *collectable, const char *desc, MVMuint64 *cache) {
     if (collectable)
         add_reference_const_cstr_cached(tc, ss, desc,
             get_collectable_idx(tc, ss, collectable), cache);
@@ -875,7 +835,7 @@ MVMuint32 get_new_toc_entry(MVMThreadContext *tc, MVMHeapDumpTableOfContents *to
     return i;
 }
 
-void serialize_attribute_stream(MVMThreadContext *tc, MVMHeapSnapshotCollection *col, char *name, char *start, size_t offset, size_t elem_size, size_t count, FILE* fh) {
+static void serialize_attribute_stream(MVMThreadContext *tc, MVMHeapSnapshotCollection *col, char *name, char *start, size_t offset, size_t elem_size, size_t count, FILE* fh) {
     char *pointer = (char *)start;
 
     ZSTD_CStream *cstream;
@@ -891,7 +851,7 @@ void serialize_attribute_stream(MVMThreadContext *tc, MVMHeapSnapshotCollection
 
     MVMuint16 elem_size_to_write = elem_size;
 
-    size_t written_total = 0;
+    /*size_t written_total = 0;*/
     /*size_t original_total = count * elem_size;*/
 
     char *out_buffer = MVM_malloc(outSize);
@@ -934,7 +894,6 @@ void serialize_attribute_stream(MVMThreadContext *tc, MVMHeapSnapshotCollection
 
 
     while (count > 0) {
-        size_t written;
         ZSTD_inBuffer inbuf;
 
         inbuf.src = pointer;
@@ -947,8 +906,7 @@ void serialize_attribute_stream(MVMThreadContext *tc, MVMHeapSnapshotCollection
         }
 
         if (outbuf.pos) {
-            written = fwrite(outbuf.dst, sizeof(char), outbuf.pos, fh);
-            written_total += written;
+            fwrite(outbuf.dst, sizeof(char), outbuf.pos, fh);
             outbuf.pos = 0;
         }
 
@@ -957,15 +915,12 @@ void serialize_attribute_stream(MVMThreadContext *tc, MVMHeapSnapshotCollection
     }
 
     do {
-        size_t written;
-
         if (ZSTD_isError(return_value = ZSTD_endStream(cstream, &outbuf))) {
             MVM_panic(1, "ZSTD compression error in heap snapshot: %s", ZSTD_getErrorName(return_value));
         }
 
         if (outbuf.pos) {
-            written = fwrite(outbuf.dst, sizeof(char), outbuf.pos, fh);
-            written_total += written;
+            fwrite(outbuf.dst, sizeof(char), outbuf.pos, fh);
             outbuf.pos = 0;
         }
     } while (return_value != 0 && !ZSTD_isError(return_value));
@@ -1003,7 +958,7 @@ void string_heap_to_filehandle_ver3(MVMThreadContext *tc, MVMHeapSnapshotCollect
     ZSTD_inBuffer inbuf;
     ZSTD_outBuffer outbuf;
 
-    char typename[8] = "strings\0";
+    char typename[9] = "strings\0";
     MVMuint64 size = 0;
 
     MVMuint64 size_position = 0;
@@ -1035,7 +990,7 @@ void string_heap_to_filehandle_ver3(MVMThreadContext *tc, MVMHeapSnapshotCollect
     }
 
     size_position = ftell(fh);
-    fwrite(&typename, sizeof(char), 8, fh);
+    fwrite(&typename, sizeof(char), sizeof(typename) - 1, fh);
     /* If the stream is seekable, we could go back here to write the size,
      * but only if we don't write everything in one go anyway */
 
@@ -1411,7 +1366,7 @@ void snapshot_to_filehandle_ver3(MVMThreadContext *tc, MVMHeapSnapshotCollection
 }
 #endif /* heapsnapshot version 3 */
 
-void static_frames_to_filehandle_ver2(MVMThreadContext *tc, MVMHeapSnapshotCollection *col) {
+static void static_frames_to_filehandle_ver2(MVMThreadContext *tc, MVMHeapSnapshotCollection *col) {
     MVMuint64 i;
     MVMHeapDumpIndex *index = col->index;
     FILE *fh = col->fh;
@@ -1460,7 +1415,7 @@ void static_frames_to_filehandle_ver2(MVMThreadContext *tc, MVMHeapSnapshotColle
 }
 
 
-void string_heap_to_filehandle_ver2(MVMThreadContext *tc, MVMHeapSnapshotCollection *col) {
+static void string_heap_to_filehandle_ver2(MVMThreadContext *tc, MVMHeapSnapshotCollection *col) {
     MVMuint64 i;
     MVMHeapDumpIndex *index = col->index;
     FILE *fh = col->fh;
@@ -1519,7 +1474,7 @@ void string_heap_to_filehandle_ver2(MVMThreadContext *tc, MVMHeapSnapshotCollect
  * so that the parser can confidently skip to the exact middle of
  * the references table and parse it with two threads in parallel.
  */
-void references_to_filehandle_ver2(MVMThreadContext *tc, MVMHeapSnapshotCollection *col, MVMHeapDumpIndexSnapshotEntry *entry) {
+static void references_to_filehandle_ver2(MVMThreadContext *tc, MVMHeapSnapshotCollection *col, MVMHeapDumpIndexSnapshotEntry *entry) {
     MVMuint64 i;
     MVMuint64 halfway;
     FILE *fh = col->fh;
@@ -1614,7 +1569,7 @@ void references_to_filehandle_ver2(MVMThreadContext *tc, MVMHeapSnapshotCollecti
  * We also write a partial table after every snapshot so that if the process
  * crashes we still have a usable file.
  */
-void types_to_filehandle_ver2(MVMThreadContext *tc, MVMHeapSnapshotCollection *col) {
+static void types_to_filehandle_ver2(MVMThreadContext *tc, MVMHeapSnapshotCollection *col) {
     MVMuint64 i;
     MVMHeapDumpIndex *index = col->index;
     FILE *fh = col->fh;
@@ -1660,7 +1615,7 @@ void types_to_filehandle_ver2(MVMThreadContext *tc, MVMHeapSnapshotCollection *c
  * This sizes table has three entries for each entry. The first one is the
  * size of the collectables table.
  */
-void collectables_to_filehandle_ver2(MVMThreadContext *tc, MVMHeapSnapshotCollection *col, MVMHeapDumpIndexSnapshotEntry *entry) {
+static void collectables_to_filehandle_ver2(MVMThreadContext *tc, MVMHeapSnapshotCollection *col, MVMHeapDumpIndexSnapshotEntry *entry) {
     MVMuint64 i;
     FILE *fh = col->fh;
     MVMHeapSnapshot *s = col->snapshot;
@@ -1719,7 +1674,7 @@ void collectables_to_filehandle_ver2(MVMThreadContext *tc, MVMHeapSnapshotCollec
 }
 
 
-void snapshot_to_filehandle_ver2(MVMThreadContext *tc, MVMHeapSnapshotCollection *col) {
+static void snapshot_to_filehandle_ver2(MVMThreadContext *tc, MVMHeapSnapshotCollection *col) {
     MVMHeapDumpIndex *index = col->index;
     MVMuint64 i = col->snapshot_idx;
     MVMHeapDumpIndexSnapshotEntry *entry;
@@ -1745,7 +1700,7 @@ void snapshot_to_filehandle_ver2(MVMThreadContext *tc, MVMHeapSnapshotCollection
 
     /*entry->incremental_data = index->stringheap_size + index->types_size + index->staticframes_size;*/
 }
-void index_to_filehandle(MVMThreadContext *tc, MVMHeapSnapshotCollection *col) {
+static void index_to_filehandle(MVMThreadContext *tc, MVMHeapSnapshotCollection *col) {
     MVMHeapDumpIndex *index = col->index;
     FILE *fh = col->fh;
 
@@ -1764,7 +1719,7 @@ static void filemeta_to_filehandle_ver3(MVMThreadContext *tc, MVMHeapSnapshotCol
     MVMuint64 size;
     FILE *fh = col->fh;
 
-    char typename[8] = "filemeta";
+    char typename[9] = "filemeta";
 
     snprintf(metadata, 1023,
             "{ "
@@ -1788,7 +1743,7 @@ static void filemeta_to_filehandle_ver3(MVMThreadContext *tc, MVMHeapSnapshotCol
     size = strlen(metadata) + 1;
 
     size_position = ftell(fh);
-    fwrite(&typename, sizeof(char), 8, fh);
+    fwrite(&typename, sizeof(char), sizeof(typename) - 1, fh);
     fwrite(&size, sizeof(MVMuint64), 1, fh);
 
     fputs(metadata, fh);
@@ -1813,7 +1768,7 @@ static void snapmeta_to_filehandle_ver3(MVMThreadContext *tc, MVMHeapSnapshotCol
     MVMuint64 size;
     FILE *fh = col->fh;
 
-    char typename[8] = "snapmeta";
+    char typename[9] = "snapmeta";
 
     snprintf(metadata, 1023,
             "{ "
@@ -1840,7 +1795,7 @@ static void snapmeta_to_filehandle_ver3(MVMThreadContext *tc, MVMHeapSnapshotCol
     size = strlen(metadata) + 1;
 
     size_position = ftell(fh);
-    fwrite(&typename, sizeof(char), 8, fh);
+    fwrite(&typename, sizeof(char), sizeof(typename) - 1, fh);
     fwrite(&size, sizeof(MVMuint64), 1, fh);
 
     fputs(metadata, fh);
@@ -1858,7 +1813,7 @@ static void snapmeta_to_filehandle_ver3(MVMThreadContext *tc, MVMHeapSnapshotCol
 }
 #endif
 
-void finish_collection_to_filehandle(MVMThreadContext *tc, MVMHeapSnapshotCollection *col) {
+static void finish_collection_to_filehandle(MVMThreadContext *tc, MVMHeapSnapshotCollection *col) {
     /*col->strings_written = 0;*/
     /*col->types_written = 0;*/
     /*col->static_frames_written = 0;*/
