@@ -899,15 +899,21 @@ static int translate_dispatch_program(MVMThreadContext *tc, MVMSpeshGraph *g,
                 MVMSpeshOperand constint = { .lit_i64 = dp->constants[op->load.idx].i64 };
                 emit_bi_op(tc, g, bb, &insert_after, MVM_OP_const_i64,
                     temporaries[op->load.temp], constint);
+                MVMSpeshFacts *f = MVM_spesh_get_facts(tc, g, temp);
+                f->flags |= MVM_SPESH_FACT_KNOWN_VALUE;
+                f->value.i = constint.lit_i64;
                 break;
             }
             case MVMDispOpcodeLoadConstantNum: {
                 MVMSpeshOperand temp = MVM_spesh_manipulate_get_temp_reg(tc, g, MVM_reg_num64);
                 temporaries[op->load.temp] = temp;
                 MVM_VECTOR_PUSH(allocated_temps, temp);
-                MVMSpeshOperand constnum = { .lit_i64 = dp->constants[op->load.idx].n64 };
+                MVMSpeshOperand constnum = { .lit_n64 = dp->constants[op->load.idx].n64 };
                 emit_bi_op(tc, g, bb, &insert_after, MVM_OP_const_n64,
                     temporaries[op->load.temp], constnum);
+                MVMSpeshFacts *f = MVM_spesh_get_facts(tc, g, temp);
+                f->flags |= MVM_SPESH_FACT_KNOWN_VALUE;
+                f->value.n = constnum.lit_n64;
                 break;
             }
             case MVMDispOpcodeLoadAttributeObj: {
@@ -1190,6 +1196,14 @@ static int translate_dispatch_program(MVMThreadContext *tc, MVMSpeshGraph *g,
                             base_op = MVM_op_get_op(MVM_OP_sp_runnativecall_i);
                             box_return_value = 1;
                             break;
+                        case MVM_NATIVECALL_ARG_UCHAR:
+                        case MVM_NATIVECALL_ARG_USHORT:
+                        case MVM_NATIVECALL_ARG_UINT:
+                        case MVM_NATIVECALL_ARG_ULONG:
+                        case MVM_NATIVECALL_ARG_ULONGLONG:
+                            base_op = MVM_op_get_op(MVM_OP_sp_runnativecall_u);
+                            box_return_value = 1;
+                            break;
                     }
                 }
 
@@ -1631,8 +1645,9 @@ int MVM_spesh_disp_optimize(MVMThreadContext *tc, MVMSpeshGraph *g, MVMSpeshBB *
                     }
                 }
             }
-            qsort(outcome_hits, MVM_VECTOR_ELEMS(outcome_hits), sizeof(OutcomeHitCount),
-                compare_hits);
+            if (outcome_hits != NULL)
+                qsort(outcome_hits, MVM_VECTOR_ELEMS(outcome_hits), sizeof(OutcomeHitCount),
+                    compare_hits);
             MVMint32 selected_outcome = -1;
             if (MVM_VECTOR_ELEMS(outcome_hits) == 0) {
                 MVM_spesh_graph_add_comment(tc, g, ins, p
