@@ -40,6 +40,54 @@ void MVM_spesh_usages_add_for_handler_by_reg(MVMThreadContext *tc, MVMSpeshGraph
     MVM_spesh_usages_add_for_handler(tc, g, MVM_spesh_get_facts(tc, g, used));
 }
 
+/* Takes a spesh graph and adds usage information. */
+static void add_usage_for_bb(MVMThreadContext *tc, MVMSpeshGraph *g, MVMSpeshBB *bb) {
+    MVMint32 i;
+    MVMSpeshIns *ins = bb->first_ins;
+    while (ins) {
+        /* Look through operands for reads and writes. */
+        MVMint32 is_phi = ins->info->opcode == MVM_SSA_PHI;
+        for (i = 0; i < ins->info->num_operands; i++) {
+            /* Reads need usage tracking. */
+            if ((is_phi && i > 0)
+                || (!is_phi && (ins->info->operands[i] & MVM_operand_rw_mask) == MVM_operand_read_reg)) {
+                MVMSpeshFacts *facts = &(g->facts[ins->operands[i].reg.orig][ins->operands[i].reg.i]);
+                MVM_spesh_usages_add(tc, g, facts, ins);
+            }
+
+            /* Writes need the writing instruction to be specified. */
+            if ((is_phi && i == 0)
+                || (!is_phi && (ins->info->operands[i] & MVM_operand_rw_mask) == MVM_operand_write_reg)) {
+                MVMSpeshFacts *facts = &(g->facts[ins->operands[i].reg.orig][ins->operands[i].reg.i]);
+                facts->writer    = ins;
+            }
+        }
+
+        /* These all read as well as write a value, so bump usages. */
+        switch (ins->info->opcode) {
+            case MVM_OP_inc_i:
+            case MVM_OP_inc_u:
+            case MVM_OP_dec_i:
+            case MVM_OP_dec_u: {
+                MVMSpeshOperand reader;
+                reader.reg.orig = ins->operands[0].reg.orig;
+                reader.reg.i = ins->operands[0].reg.i - 1;
+                MVM_spesh_usages_add_by_reg(tc, g, reader, ins);
+                break;
+            }
+        }
+
+        ins = ins->next;
+    }
+
+    /* Visit children. */
+    for (i = 0; i < bb->num_children; i++)
+        add_usage_for_bb(tc, g, bb->children[i]);
+}
+void MVM_spesh_usages_create_usage(MVMThreadContext *tc, MVMSpeshGraph *g) {
+    add_usage_for_bb(tc, g, g->entry);
+}
+
 /* Takes a spesh graph with DU chains already build and correct, and builds up
  * deopt use chains for it. The algorithm keeps track of writers with reads
  * that have not yet been observed, and when we reach a deopt point adds the
@@ -168,11 +216,30 @@ static void process_bb_for_deopt_usage(MVMThreadContext *tc, DeoptAnalysisState
     /* Walk the BB's instructions. */
     MVMSpeshIns *ins = bb->first_ins;
     while (ins) {
-        MVMint16 opcode = ins->info->opcode;
-        MVMuint8 is_phi = opcode == MVM_SSA_PHI;
-        MVMSpeshAnn *ann;
+        /* Work out of it's a deopt point. If it's a pre-deopt point, we should
+         * immediately process it as the values that are read by the op must be
+         * available to it after deopt. */
+        MVMSpeshAnn *ann = ins->annotations;
+        MVMSpeshAnn *deopt_ann = NULL;
+        MVMSpeshAnn *deopt_all_ann = NULL;
+        while (ann) {
+            switch (ann->type) {
+                case MVM_SPESH_ANN_DEOPT_PRE_INS:
+                    process_deopt(tc, state, g, bb, ins, ann->data.deopt_idx);
+                    break;
+                case MVM_SPESH_ANN_DEOPT_ONE_INS:
+                    deopt_ann = ann;
+                    break;
+                case MVM_SPESH_ANN_DEOPT_ALL_INS:
+                    deopt_all_ann = ann;
+                    break;
+            }
+            ann = ann->next;
+        }
 
         /* Process the read operands. */
+        MVMint16 opcode = ins->info->opcode;
+        MVMuint8 is_phi = opcode == MVM_SSA_PHI;
         if (MVM_spesh_is_inc_dec_op(opcode)) {
             MVMSpeshOperand read = ins->operands[0];
             read.reg.i--;
@@ -188,17 +255,12 @@ static void process_bb_for_deopt_usage(MVMThreadContext *tc, DeoptAnalysisState
                     process_read(tc, state, g, bb, ins, ins->operands[i]);
         }
 
-        /* If it's a deopt point, add currently unread writes as dependencies. */
-        ann = ins->annotations;
-        while (ann) {
-            switch (ann->type) {
-                case MVM_SPESH_ANN_DEOPT_ONE_INS:
-                case MVM_SPESH_ANN_DEOPT_ALL_INS:
-                    process_deopt(tc, state, g, bb, ins, ann->data.deopt_idx);
-                    break;
-            }
-            ann = ann->next;
-        }
+        /* Now we've processed the reads, a post-deopt point should be put
+         * into effect. */
+        if (deopt_ann)
+            process_deopt(tc, state, g, bb, ins, deopt_ann->data.deopt_idx);
+        if (deopt_all_ann)
+            process_deopt(tc, state, g, bb, ins, deopt_all_ann->data.deopt_idx);
 
         /* Process the write. */
         if (is_phi) {
@@ -271,6 +333,19 @@ void MVM_spesh_usages_create_deopt_usage(MVMThreadContext *tc, MVMSpeshGraph *g)
     process_bb_for_deopt_usage(tc, &state, g, g->entry);
 }
 
+/* Adds a deopt usage. */
+void MVM_spesh_usages_add_deopt_usage(MVMThreadContext *tc, MVMSpeshGraph *g,
+        MVMSpeshFacts *facts, MVMint32 deopt_idx) {
+    MVMSpeshDeoptUseEntry *deopt_entry = MVM_spesh_alloc(tc, g, sizeof(MVMSpeshDeoptUseEntry));
+    deopt_entry->deopt_idx = deopt_idx;
+    deopt_entry->next = facts->usage.deopt_users;
+    facts->usage.deopt_users = deopt_entry;
+}
+void MVM_spesh_usages_add_deopt_usage_by_reg(MVMThreadContext *tc, MVMSpeshGraph *g,
+        MVMSpeshOperand operand, MVMint32 deopt_idx) {
+    MVM_spesh_usages_add_deopt_usage(tc, g, MVM_spesh_get_facts(tc, g, operand), deopt_idx);
+}
+
 /* Adds an unconditional deopt usage (that is, not dependent on any particular
  * deopt point). */
 void MVM_spesh_usages_add_unconditional_deopt_usage(MVMThreadContext *tc, MVMSpeshGraph *g,
@@ -305,23 +380,23 @@ void MVM_spesh_usages_remove_unused_deopt(MVMThreadContext *tc, MVMSpeshGraph *g
     MVMuint8 *deopt_used = MVM_spesh_alloc(tc, g, g->num_deopt_addrs);
     MVMSpeshBB *bb = g->entry;
     while (bb) {
-        if (!bb->inlined) {
-            MVMSpeshIns *ins = bb->first_ins;
-            while (ins) {
-                MVMSpeshAnn *ann = ins->annotations;
-                while (ann) {
-                    switch (ann->type) {
-                        case MVM_SPESH_ANN_DEOPT_ONE_INS:
-                        case MVM_SPESH_ANN_DEOPT_ALL_INS:
-                        case MVM_SPESH_ANN_DEOPT_SYNTH:
-                            if (ins->info->may_cause_deopt)
-                                deopt_used[ann->data.deopt_idx] = 1;
-                            break;
-                    }
-                    ann = ann->next;
+        MVMSpeshIns *ins = bb->first_ins;
+        while (ins) {
+            MVMSpeshAnn *ann = ins->annotations;
+            while (ann) {
+                switch (ann->type) {
+                    case MVM_SPESH_ANN_DEOPT_ONE_INS:
+                    case MVM_SPESH_ANN_DEOPT_PRE_INS:
+                    case MVM_SPESH_ANN_DEOPT_ALL_INS:
+                    case MVM_SPESH_ANN_DEOPT_INLINE:
+                    case MVM_SPESH_ANN_DEOPT_SYNTH:
+                        if (ins->info->may_cause_deopt)
+                            deopt_used[ann->data.deopt_idx] = 1;
+                        break;
                 }
-                ins = ins->next;
+                ann = ann->next;
             }
+            ins = ins->next;
         }
         bb = bb->linear_next;
     }
